@@ -1,6 +1,6 @@
 # -*- encoding: utf-8 -*-
 #
-# Copyright © 2014-2015 eNovance
+# Copyright © 2014-2016 eNovance
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -13,14 +13,15 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-import errno
 import functools
 import json
 import os
+import subprocess
 import uuid
 
 import fixtures
 from oslotest import base
+from oslotest import mockpatch
 import six
 from six.moves.urllib.parse import unquote
 try:
@@ -57,206 +58,6 @@ def _skip_decorator(func):
     return skip_if_not_implemented
 
 
-class FakeRadosModule(object):
-    class OpCtx(object):
-        def __enter__(self):
-            self.ops = []
-            return self
-
-        def __exit__(self, *args, **kwargs):
-            pass
-
-    WriteOpCtx = ReadOpCtx = OpCtx
-
-    class OmapIterator(object):
-        class OpRetCode(object):
-            def __init__(self):
-                self.ret = 0
-
-            def __eq__(self, other):
-                return self.ret == other
-
-        def __init__(self, start_filter, prefix_filter, number):
-            self.start_filter = start_filter
-            self.prefix_filter = prefix_filter
-            self.number = number
-            self.data = {}
-            self.op_ret = self.OpRetCode()
-
-        def set_data(self, data):
-            if not data:
-                self.op_ret.ret = errno.ENOENT
-            else:
-                self.data = data
-
-        def __iter__(self):
-            # NOTE(sileht): we use only the prefix for now
-            return ((k, v) for k, v in self.data.items()
-                    if k.startswith(self.prefix_filter))
-
-    LIBRADOS_OPERATION_BALANCE_READS = 1
-    LIBRADOS_OPERATION_SKIPRWLOCKS = 16
-
-    class ObjectNotFound(Exception):
-        pass
-
-    class ioctx(object):
-        def __init__(self, kvs, kvs_xattrs, kvs_omaps):
-            self.kvs = kvs
-            self.kvs_xattrs = kvs_xattrs
-            self.kvs_omaps = kvs_omaps
-            self.librados = self
-            self.io = self
-
-        def __enter__(self):
-            return self
-
-        @staticmethod
-        def __exit__(exc_type, exc_value, traceback):
-            pass
-
-        def _ensure_key_exists(self, key):
-            if key not in self.kvs:
-                self.kvs[key] = ""
-                self.kvs_xattrs[key] = {}
-                self.kvs_omaps[key] = {}
-
-        @staticmethod
-        def close():
-            pass
-
-        @staticmethod
-        def _validate_key(name):
-            if not isinstance(name, str):
-                raise TypeError("key is not a 'str' object")
-
-        def write_full(self, key, value):
-            self._validate_key(key)
-            self._ensure_key_exists(key)
-            self.kvs[key] = value
-
-        def write(self, key, value, offset):
-            self._validate_key(key)
-            try:
-                current = self.kvs[key]
-            except KeyError:
-                current = b""
-            if len(current) < offset:
-                current += b'\x00' * (offset - len(current))
-            self.kvs[key] = (
-                current[:offset] + value + current[offset + len(value):]
-            )
-
-        def stat(self, key):
-            self._validate_key(key)
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            else:
-                return (1024, "timestamp")
-
-        def read(self, key, length=8192, offset=0):
-            self._validate_key(key)
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            else:
-                return self.kvs[key][offset:offset+length]
-
-        def operate_read_op(self, op, key, flag=0):
-            for op in op.ops:
-                op(key)
-
-        def get_omap_vals(self, op, start_filter, prefix_filter, number):
-            oi = FakeRadosModule.OmapIterator(start_filter, prefix_filter,
-                                              number)
-            op.ops.append(lambda oid: oi.set_data(self.kvs_omaps.get(oid)))
-            return oi, oi.op_ret
-
-        def operate_write_op(self, op, key, flags=0):
-            for op in op.ops:
-                op(key)
-
-        def set_omap(self, op,  keys, values):
-            def add(oid):
-                self._ensure_key_exists(oid)
-                omaps = self.kvs_omaps.setdefault(oid, {})
-                omaps.update(dict(zip(keys, values)))
-            op.ops.append(add)
-
-        def remove_omap_keys(self, op,  keys):
-            def rm(oid):
-                for key in keys:
-                    del self.kvs_omaps[oid][key]
-            op.ops.append(rm)
-
-        def get_xattrs(self, key):
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            return six.iteritems(self.kvs_xattrs.get(key, {}).copy())
-
-        def set_xattr(self, key, attr, value):
-            self._ensure_key_exists(key)
-            xattrs = self.kvs_xattrs.setdefault(key, {})
-            xattrs[attr] = value
-
-        def rm_xattr(self, key, attr):
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            del self.kvs_xattrs[key][attr]
-
-        def remove_object(self, key):
-            self._validate_key(key)
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            del self.kvs[key]
-            del self.kvs_xattrs[key]
-            del self.kvs_omaps[key]
-
-        def aio_remove(self, key):
-            self._validate_key(key)
-            self.kvs.pop(key, None)
-            self.kvs_xattrs.pop(key, None)
-            self.kvs_omaps.pop(key, None)
-
-        @staticmethod
-        def aio_flush():
-            pass
-
-    class FakeRados(object):
-        def __init__(self, kvs, kvs_xattrs, kvs_omaps):
-            self.kvs = kvs
-            self.kvs_xattrs = kvs_xattrs
-            self.kvs_omaps = kvs_omaps
-
-        @staticmethod
-        def connect():
-            pass
-
-        @staticmethod
-        def shutdown():
-            pass
-
-        def open_ioctx(self, pool):
-            return FakeRadosModule.ioctx(self.kvs, self.kvs_xattrs,
-                                         self.kvs_omaps)
-
-    def __init__(self):
-        self.kvs = {}
-        self.kvs_xattrs = {}
-        self.kvs_omaps = {}
-
-    def Rados(self, *args, **kwargs):
-        return FakeRadosModule.FakeRados(self.kvs, self.kvs_xattrs,
-                                         self.kvs_omaps)
-
-    @staticmethod
-    def run_in_thread(method, args):
-        return method(*args)
-
-    @staticmethod
-    def make_ex(ret, reason):
-        raise Exception(reason)
-
-
 class FakeSwiftClient(object):
     def __init__(self, *args, **kwargs):
         self.kvs = {}
@@ -425,25 +226,21 @@ class TestCase(base.BaseTestCase):
         ),
     }
 
-    @staticmethod
-    def path_get(project_file=None):
-        root = os.path.abspath(os.path.join(os.path.dirname(__file__),
-                                            '..',
-                                            '..',
-                                            )
-                               )
-        if project_file:
-            return os.path.join(root, project_file)
-        return root
-
     @classmethod
     def setUpClass(self):
         super(TestCase, self).setUpClass()
         self.conf = service.prepare_service([],
                                             default_config_files=[])
-        self.conf.set_override('policy_file',
-                               self.path_get('etc/gnocchi/policy.json'),
-                               group="oslo_policy")
+
+        # NOTE(jd) This allows to test S3 on AWS
+        if not os.getenv("AWS_ACCESS_KEY_ID"):
+            self.conf.set_override('s3_endpoint_url',
+                                   os.getenv("GNOCCHI_STORAGE_HTTP_URL"),
+                                   group="storage")
+            self.conf.set_override('s3_access_key_id', "gnocchi",
+                                   group="storage")
+            self.conf.set_override('s3_secret_access_key', "anythingworks",
+                                   group="storage")
 
         self.index = indexer.get_driver(self.conf)
         self.index.connect()
@@ -472,26 +269,30 @@ class TestCase(base.BaseTestCase):
             except indexer.ArchivePolicyAlreadyExists:
                 pass
 
-        self.conf.set_override(
-            'driver',
-            os.getenv("GNOCCHI_TEST_STORAGE_DRIVER", "file"),
-            'storage')
+        storage_driver = os.getenv("GNOCCHI_TEST_STORAGE_DRIVER", "file")
+        self.conf.set_override('driver', storage_driver, 'storage')
+        if storage_driver == 'ceph':
+            self.conf.set_override('ceph_conffile',
+                                   os.getenv("CEPH_CONF"),
+                                   'storage')
 
     def setUp(self):
         super(TestCase, self).setUp()
         if swexc:
-            self.useFixture(fixtures.MockPatch(
+            self.useFixture(mockpatch.Patch(
                 'swiftclient.client.Connection',
                 FakeSwiftClient))
 
-        self.useFixture(fixtures.MockPatch('gnocchi.storage.ceph.rados',
-                                           FakeRadosModule()))
-
         if self.conf.storage.driver == 'file':
             tempdir = self.useFixture(fixtures.TempDir())
             self.conf.set_override('file_basepath',
                                    tempdir.path,
                                    'storage')
+        elif self.conf.storage.driver == 'ceph':
+            pool_name = uuid.uuid4().hex
+            subprocess.call("rados -c %s mkpool %s" % (
+                os.getenv("CEPH_CONF"), pool_name), shell=True)
+            self.conf.set_override('ceph_pool', pool_name, 'storage')
 
         self.storage = storage.get_driver(self.conf)
         # NOTE(jd) Do not upgrade the storage. We don't really need the storage
