@@ -40,7 +40,6 @@ class TestStorageDriver(tests_base.TestCase):
         m = storage.Metric(uuid.uuid4(),
                            self.archive_policies[archive_policy_name])
         m_sql = self.index.create_metric(m.id, str(uuid.uuid4()),
-                                         str(uuid.uuid4()),
                                          archive_policy_name)
         return m, m_sql
 
@@ -56,14 +55,18 @@ class TestStorageDriver(tests_base.TestCase):
     def test_corrupted_data(self):
         if not isinstance(self.storage, _carbonara.CarbonaraBasedStorage):
             self.skipTest("This driver is not based on Carbonara")
+        if self.conf.storage.driver == "s3":
+            self.skipTest(
+                "This test does not work with S3 as backend as the S3 driver "
+                "has no fake client, and tests run in parallel.")
 
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
         ])
         self.trigger_processing()
 
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 13, 0, 1), 1),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 13, 0, 1), 1),
         ])
 
         with mock.patch('gnocchi.carbonara.AggregatedTimeSerie.unserialize',
@@ -78,38 +81,42 @@ class TestStorageDriver(tests_base.TestCase):
         self.assertIn((utils.datetime_utc(2014, 1, 1, 13), 300.0, 1), m)
 
     def test_list_metric_with_measures_to_process(self):
-        metrics = self.storage.list_metric_with_measures_to_process(
+        if self.conf.storage.driver == "s3":
+            self.skipTest(
+                "This test does not work with S3 as backend as the S3 driver "
+                "has no fake client, and tests run in parallel.")
+        metrics = self.storage.incoming.list_metric_with_measures_to_process(
             None, None, full=True)
         self.assertEqual(set(), metrics)
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
         ])
-        metrics = self.storage.list_metric_with_measures_to_process(
+        metrics = self.storage.incoming.list_metric_with_measures_to_process(
             None, None, full=True)
         self.assertEqual(set([str(self.metric.id)]), metrics)
         self.trigger_processing()
-        metrics = self.storage.list_metric_with_measures_to_process(
+        metrics = self.storage.incoming.list_metric_with_measures_to_process(
             None, None, full=True)
         self.assertEqual(set([]), metrics)
 
     def test_delete_nonempty_metric(self):
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
         ])
         self.trigger_processing()
         self.storage.delete_metric(self.metric, sync=True)
         self.trigger_processing()
 
     def test_delete_nonempty_metric_unprocessed(self):
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
         ])
         self.storage.delete_metric(self.metric, sync=True)
         self.trigger_processing()
 
     def test_delete_expunge_metric(self):
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
         ])
         self.trigger_processing()
         self.index.delete_metric(self.metric.id)
@@ -118,14 +125,14 @@ class TestStorageDriver(tests_base.TestCase):
                           self.metric.id)
 
     def test_measures_reporting(self):
-        report = self.storage.measures_report(True)
+        report = self.storage.incoming.measures_report(True)
         self.assertIsInstance(report, dict)
         self.assertIn('summary', report)
         self.assertIn('metrics', report['summary'])
         self.assertIn('measures', report['summary'])
         self.assertIn('details', report)
         self.assertIsInstance(report['details'], dict)
-        report = self.storage.measures_report(False)
+        report = self.storage.incoming.measures_report(False)
         self.assertIsInstance(report, dict)
         self.assertIn('summary', report)
         self.assertIn('metrics', report['summary'])
@@ -134,8 +141,8 @@ class TestStorageDriver(tests_base.TestCase):
 
     def test_add_measures_big(self):
         m, __ = self._create_metric('high')
-        self.storage.add_measures(m, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, i, j), 100)
+        self.storage.incoming.add_measures(m, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, i, j), 100)
             for i in six.moves.range(0, 60) for j in six.moves.range(0, 60)])
         self.trigger_processing([str(m.id)])
 
@@ -143,16 +150,20 @@ class TestStorageDriver(tests_base.TestCase):
 
     @mock.patch('gnocchi.carbonara.SplitKey.POINTS_PER_SPLIT', 48)
     def test_add_measures_update_subset_split(self):
+        if self.conf.storage.driver == "s3":
+            self.skipTest(
+                "This test does not work with S3 as backend as the S3 driver "
+                "has no fake client, and tests run in parallel.")
         m, m_sql = self._create_metric('medium')
         measures = [
-            storage.Measure(utils.datetime_utc(2014, 1, 6, i, j, 0), 100)
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 6, i, j, 0), 100)
             for i in six.moves.range(2) for j in six.moves.range(0, 60, 2)]
-        self.storage.add_measures(m, measures)
+        self.storage.incoming.add_measures(m, measures)
         self.trigger_processing([str(m.id)])
 
         # add measure to end, in same aggregate time as last point.
-        self.storage.add_measures(m, [
-            storage.Measure(utils.datetime_utc(2014, 1, 6, 1, 58, 1), 100)])
+        self.storage.incoming.add_measures(m, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 6, 1, 58, 1), 100)])
 
         with mock.patch.object(self.storage, '_store_metric_measures') as c:
             # should only resample last aggregate
@@ -166,18 +177,23 @@ class TestStorageDriver(tests_base.TestCase):
         self.assertEqual(1, count)
 
     def test_add_measures_update_subset(self):
+        if self.conf.storage.driver == "s3":
+            self.skipTest(
+                "This test does not work with S3 as backend as the S3 driver "
+                "has no fake client, and tests run in parallel.")
         m, m_sql = self._create_metric('medium')
         measures = [
-            storage.Measure(utils.datetime_utc(2014, 1, 6, i, j, 0), 100)
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 6, i, j, 0), 100)
             for i in six.moves.range(2) for j in six.moves.range(0, 60, 2)]
-        self.storage.add_measures(m, measures)
+        self.storage.incoming.add_measures(m, measures)
         self.trigger_processing([str(m.id)])
 
         # add measure to end, in same aggregate time as last point.
-        new_point = utils.datetime_utc(2014, 1, 6, 1, 58, 1)
-        self.storage.add_measures(m, [storage.Measure(new_point, 100)])
+        new_point = utils.dt_to_unix_ns(2014, 1, 6, 1, 58, 1)
+        self.storage.incoming.add_measures(
+            m, [storage.Measure(new_point, 100)])
 
-        with mock.patch.object(self.storage, '_add_measures') as c:
+        with mock.patch.object(self.storage.incoming, 'add_measures') as c:
             self.trigger_processing([str(m.id)])
         for __, args, __ in c.mock_calls:
             self.assertEqual(
@@ -185,11 +201,11 @@ class TestStorageDriver(tests_base.TestCase):
                     new_point, args[1].granularity * 10e8))
 
     def test_delete_old_measures(self):
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.trigger_processing()
 
@@ -202,8 +218,8 @@ class TestStorageDriver(tests_base.TestCase):
         ], self.storage.get_measures(self.metric))
 
         # One year later…
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2015, 1, 1, 12, 0, 1), 69),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2015, 1, 1, 12, 0, 1), 69),
         ])
         self.trigger_processing()
 
@@ -232,15 +248,14 @@ class TestStorageDriver(tests_base.TestCase):
         self.index.create_archive_policy(ap)
         self.metric = storage.Metric(uuid.uuid4(), ap)
         self.index.create_metric(self.metric.id, str(uuid.uuid4()),
-                                 str(uuid.uuid4()),
                                  apname)
 
         # First store some points scattered across different splits
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2016, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2016, 1, 2, 13, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2016, 1, 4, 14, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2016, 1, 6, 15, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 2, 13, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 4, 14, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 6, 15, 12, 45), 44),
         ])
         self.trigger_processing()
 
@@ -276,9 +291,9 @@ class TestStorageDriver(tests_base.TestCase):
         # split (keep in mind the back window size in one hour here). We move
         # the BoundTimeSerie processing timeserie far away from its current
         # range.
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2016, 1, 10, 16, 18, 45), 45),
-            storage.Measure(utils.datetime_utc(2016, 1, 10, 17, 12, 45), 46),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 10, 16, 18, 45), 45),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 10, 17, 12, 45), 46),
         ])
         self.trigger_processing()
 
@@ -319,15 +334,14 @@ class TestStorageDriver(tests_base.TestCase):
         self.index.create_archive_policy(ap)
         self.metric = storage.Metric(uuid.uuid4(), ap)
         self.index.create_metric(self.metric.id, str(uuid.uuid4()),
-                                 str(uuid.uuid4()),
                                  apname)
 
         # First store some points scattered across different splits
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2016, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2016, 1, 2, 13, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2016, 1, 4, 14, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2016, 1, 6, 15, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 2, 13, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 4, 14, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 6, 15, 12, 45), 44),
         ])
         self.trigger_processing()
 
@@ -366,8 +380,8 @@ class TestStorageDriver(tests_base.TestCase):
 
         # Here we test a special case where the oldest_mutable_timestamp will
         # be 2016-01-10TOO:OO:OO = 1452384000.0, our new split key.
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2016, 1, 10, 0, 12), 45),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 10, 0, 12), 45),
         ])
         self.trigger_processing()
 
@@ -406,15 +420,14 @@ class TestStorageDriver(tests_base.TestCase):
         self.index.create_archive_policy(ap)
         self.metric = storage.Metric(uuid.uuid4(), ap)
         self.index.create_metric(self.metric.id, str(uuid.uuid4()),
-                                 str(uuid.uuid4()),
                                  apname)
 
         # First store some points scattered across different splits
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2016, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2016, 1, 2, 13, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2016, 1, 4, 14, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2016, 1, 6, 15, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 2, 13, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 4, 14, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 6, 15, 12, 45), 44),
         ])
         self.trigger_processing()
 
@@ -456,9 +469,9 @@ class TestStorageDriver(tests_base.TestCase):
         # split (keep in mind the back window size in one hour here). We move
         # the BoundTimeSerie processing timeserie far away from its current
         # range.
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2016, 1, 10, 16, 18, 45), 45),
-            storage.Measure(utils.datetime_utc(2016, 1, 10, 17, 12, 45), 46),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 10, 16, 18, 45), 45),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 10, 17, 12, 45), 46),
         ])
         self.trigger_processing()
 
@@ -470,15 +483,14 @@ class TestStorageDriver(tests_base.TestCase):
         self.index.create_archive_policy(ap)
         self.metric = storage.Metric(uuid.uuid4(), ap)
         self.index.create_metric(self.metric.id, str(uuid.uuid4()),
-                                 str(uuid.uuid4()),
                                  apname)
 
         # First store some points scattered across different splits
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2016, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2016, 1, 2, 13, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2016, 1, 4, 14, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2016, 1, 6, 15, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 2, 13, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 4, 14, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 6, 15, 12, 45), 44),
         ])
         self.trigger_processing()
 
@@ -518,16 +530,16 @@ class TestStorageDriver(tests_base.TestCase):
         # split (keep in mind the back window size in one hour here). We move
         # the BoundTimeSerie processing timeserie far away from its current
         # range.
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2016, 1, 10, 16, 18, 45), 45),
-            storage.Measure(utils.datetime_utc(2016, 1, 10, 17, 12, 45), 46),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 10, 16, 18, 45), 45),
+            storage.Measure(utils.dt_to_unix_ns(2016, 1, 10, 17, 12, 45), 46),
         ])
         self.trigger_processing()
 
     def test_updated_measures(self):
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
         ])
         self.trigger_processing()
 
@@ -538,9 +550,9 @@ class TestStorageDriver(tests_base.TestCase):
             (utils.datetime_utc(2014, 1, 1, 12, 5), 300.0, 42.0),
         ], self.storage.get_measures(self.metric))
 
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.trigger_processing()
 
@@ -569,11 +581,11 @@ class TestStorageDriver(tests_base.TestCase):
         ], self.storage.get_measures(self.metric, aggregation='min'))
 
     def test_add_and_get_measures(self):
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.trigger_processing()
 
@@ -659,11 +671,11 @@ class TestStorageDriver(tests_base.TestCase):
                                              self.archive_policies['low'])]))
 
     def test_get_measure_unknown_aggregation(self):
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.assertRaises(storage.AggregationDoesNotExist,
                           self.storage.get_measures,
@@ -672,17 +684,17 @@ class TestStorageDriver(tests_base.TestCase):
     def test_get_cross_metric_measures_unknown_aggregation(self):
         metric2 = storage.Metric(uuid.uuid4(),
                                  self.archive_policies['low'])
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
-        self.storage.add_measures(metric2, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(metric2, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.assertRaises(storage.AggregationDoesNotExist,
                           self.storage.get_cross_metric_measures,
@@ -692,17 +704,17 @@ class TestStorageDriver(tests_base.TestCase):
     def test_get_cross_metric_measures_unknown_granularity(self):
         metric2 = storage.Metric(uuid.uuid4(),
                                  self.archive_policies['low'])
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
-        self.storage.add_measures(metric2, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(metric2, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.assertRaises(storage.GranularityDoesNotExist,
                           self.storage.get_cross_metric_measures,
@@ -712,17 +724,17 @@ class TestStorageDriver(tests_base.TestCase):
     def test_add_and_get_cross_metric_measures_different_archives(self):
         metric2 = storage.Metric(uuid.uuid4(),
                                  self.archive_policies['no_granularity_match'])
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
-        self.storage.add_measures(metric2, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(metric2, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
 
         self.assertRaises(storage.MetricUnaggregatable,
@@ -731,17 +743,17 @@ class TestStorageDriver(tests_base.TestCase):
 
     def test_add_and_get_cross_metric_measures(self):
         metric2, __ = self._create_metric()
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 44),
         ])
-        self.storage.add_measures(metric2, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 5), 9),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 41), 2),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 10, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 13, 10), 4),
+        self.storage.incoming.add_measures(metric2, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 5), 9),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 41), 2),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 10, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 13, 10), 4),
         ])
         self.trigger_processing([str(self.metric.id), str(metric2.id)])
 
@@ -766,7 +778,7 @@ class TestStorageDriver(tests_base.TestCase):
 
         values = self.storage.get_cross_metric_measures(
             [self.metric, metric2],
-            from_timestamp=utils.to_timestamp('2014-01-01 12:10:00'))
+            from_timestamp=datetime.datetime(2014, 1, 1, 12, 10, 0))
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1), 86400.0, 22.25),
             (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 22.25),
@@ -775,7 +787,7 @@ class TestStorageDriver(tests_base.TestCase):
 
         values = self.storage.get_cross_metric_measures(
             [self.metric, metric2],
-            to_timestamp=utils.to_timestamp('2014-01-01 12:05:00'))
+            to_timestamp=datetime.datetime(2014, 1, 1, 12, 5, 0))
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1, 0, 0, 0), 86400.0, 22.25),
@@ -785,8 +797,8 @@ class TestStorageDriver(tests_base.TestCase):
 
         values = self.storage.get_cross_metric_measures(
             [self.metric, metric2],
-            to_timestamp=utils.to_timestamp('2014-01-01 12:10:10'),
-            from_timestamp=utils.to_timestamp('2014-01-01 12:10:10'))
+            from_timestamp=datetime.datetime(2014, 1, 1, 12, 10, 10),
+            to_timestamp=datetime.datetime(2014, 1, 1, 12, 10, 10))
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1), 86400.0, 22.25),
             (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 22.25),
@@ -795,8 +807,8 @@ class TestStorageDriver(tests_base.TestCase):
 
         values = self.storage.get_cross_metric_measures(
             [self.metric, metric2],
-            from_timestamp=utils.to_timestamp('2014-01-01 12:00:00'),
-            to_timestamp=utils.to_timestamp('2014-01-01 12:00:01'))
+            from_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 0),
+            to_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 1))
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1), 86400.0, 22.25),
@@ -806,8 +818,8 @@ class TestStorageDriver(tests_base.TestCase):
 
         values = self.storage.get_cross_metric_measures(
             [self.metric, metric2],
-            from_timestamp=utils.to_timestamp('2014-01-01 12:00:00'),
-            to_timestamp=utils.to_timestamp('2014-01-01 12:00:01'),
+            from_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 0),
+            to_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 1),
             granularity=300.0)
 
         self.assertEqual([
@@ -816,18 +828,18 @@ class TestStorageDriver(tests_base.TestCase):
 
     def test_add_and_get_cross_metric_measures_with_holes(self):
         metric2, __ = self._create_metric()
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 5, 31), 8),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 42),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 5, 31), 8),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 42),
         ])
-        self.storage.add_measures(metric2, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 5), 9),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 2),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 6),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 13, 10), 2),
+        self.storage.incoming.add_measures(metric2, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 5), 9),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 2),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 6),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 13, 10), 2),
         ])
         self.trigger_processing([str(self.metric.id), str(metric2.id)])
 
@@ -842,19 +854,19 @@ class TestStorageDriver(tests_base.TestCase):
 
     def test_search_value(self):
         metric2, __ = self._create_metric()
-        self.storage.add_measures(self.metric, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1,), 69),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 5, 31), 8),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 42),
+        self.storage.incoming.add_measures(self.metric, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 1,), 69),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 5, 31), 8),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 12, 45), 42),
         ])
 
-        self.storage.add_measures(metric2, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 5), 9),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 2),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 6),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 13, 10), 2),
+        self.storage.incoming.add_measures(metric2, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 5), 9),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 7, 31), 2),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 9, 31), 6),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 13, 10), 2),
         ])
         self.trigger_processing([str(self.metric.id), str(metric2.id)])
 
@@ -881,13 +893,12 @@ class TestStorageDriver(tests_base.TestCase):
         name = str(uuid.uuid4())
         ap = archive_policy.ArchivePolicy(name, 0, [(3, 5)])
         self.index.create_archive_policy(ap)
-        m = self.index.create_metric(uuid.uuid4(), str(uuid.uuid4()),
-                                     str(uuid.uuid4()), name)
+        m = self.index.create_metric(uuid.uuid4(), str(uuid.uuid4()), name)
         m = self.index.list_metrics(ids=[m.id])[0]
-        self.storage.add_measures(m, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 0), 1),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 5), 1),
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 10), 1),
+        self.storage.incoming.add_measures(m, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 0), 1),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 5), 1),
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 10), 1),
         ])
         self.trigger_processing([str(m.id)])
         self.assertEqual([
@@ -899,8 +910,8 @@ class TestStorageDriver(tests_base.TestCase):
         self.index.update_archive_policy(
             name, [archive_policy.ArchivePolicyItem(granularity=5, points=6)])
         m = self.index.list_metrics(ids=[m.id])[0]
-        self.storage.add_measures(m, [
-            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 15), 1),
+        self.storage.incoming.add_measures(m, [
+            storage.Measure(utils.dt_to_unix_ns(2014, 1, 1, 12, 0, 15), 1),
         ])
         self.trigger_processing([str(m.id)])
         self.assertEqual([
